/*
 * main.c - v1.4 - 12 Feb 1999
 * (c) Stevie Strickland, 1999
 *
 * This program has been placed under the GPL.  Any bugfixes or enhancements
 * will be greatly appreciated :)
 *
 * Stevie Strickland - sstrickl@resnet.gatech.edu
 */

#include "rolldice.h"

// The version number :)
static const int MAJOR_VERSION = 1;
static const int MINOR_VERSION = 4;

// Flags to use to test to see which random number method to use
int use_rand = TRUE_VAL;
int ran_file = FALSE_VAL;
int urandom = FALSE_VAL;

/*
 * print_usage() - Prints the usage for rolldice
 *
 * Parameters: none
 * Returns: none
 */
static void print_usage() {
  fprintf(stderr, "Usage: rolldice [-v|--version] [-r] [-u] <dice string>\n");
  fprintf(stderr, "For dice string format, see rolldice(6).\n");
  exit(EXIT_FAILURE);
}

/*
 * print_version() - Prints the version number
 *
 * Parameters: none
 * Returns: none
 */
static void print_version() {
    printf("rolldice, v%d.%d\n", MAJOR_VERSION, MINOR_VERSION);
    printf("Written by Stevie Strickland (sstrickl@resnet.gatech.edu)\n");
    exit(EXIT_SUCCESS);
}

/*
 * print_opt_error() - Prints out the error for an unknown option
 *
 * Parameters: option - unknown option to print out
 * Returns: none
 */
static void print_opt_error(char *option) {
  fprintf(stderr, "%s: Unknown option\n", option);
  print_usage();
}

/*
 * parse_options() - Parse the command-line options
 *
 * Parameters: String to parse for options
 * Returns: none
 */
static void parse_options(char *option) {

  option++;

  if(*option == '-') {
    option++;
    if((strcmp(option, "version")) == 0) print_version();
    else print_opt_error(option);
  }
  else 
    switch(*option) {
    case 'v': print_version();
    case 'u': urandom = TRUE_VAL; use_rand = ran_file = FALSE_VAL; break;
    case 'r': ran_file = TRUE_VAL; use_rand = urandom = FALSE_VAL; break;
    default: print_opt_error(option);
    }
}

int main(int argc, char **argv) {
 
  int index, *totals, *dice_nums;

  dice_nums = NULL;
  
  if ( argc < 2 ) {
    print_usage();
  } else {
    for ( index = 1; index < argc; index++ ) {
      if ( *argv[index] == '-' ) parse_options(argv[index]);
      else dice_nums = parse_string( argv[index] );
    }
  }

  if(use_rand)
    init_random();
  else
    open_random();

  if ( dice_nums == NULL ) {
    fprintf(stderr, "%s: Failure in getting dice attributes\n", argv[0]);
    return 1;
  }

  totals = rolldice(dice_nums);

  if( totals == NULL ) {
    fprintf(stderr, "%s: Failure in rolling dice\n", argv[0]);
    free(dice_nums);
    return 1;
  }

  for(index = 0; index < dice_nums[NUM_ROLLS]; index++)
    printf("%d ", totals[index]);

  printf("\n");

  free(totals);
  free(dice_nums);

  return EXIT_SUCCESS;
}
