/**
 * rolldice.c - v1.4 - 12 Feb 1999
 * (c) Stevie Strickland, 1999
 * 
 * This program has been placed under the GPL.  Any bugfixes or enhancements
 * will be greatly appreciated :)
 *
 * Stevie Strickland - sstrickl@resnet.gatech.edu
 */

#include "rolldice.h"

// File pointer for random device
static FILE* ran_dev;

// Flags to use to test to see which random number method to use
extern int use_rand;
extern int ran_file;
extern int urandom;

/* init_random() - Initializes the random number generator
 * 
 * Parameters: None
 * Returns: Nothing
 */
void init_random() {
  time_t t;

  (void)time(&t);

  srand((unsigned int)t);
}

void open_random() {
  if(ran_file) {
    if((ran_dev = fopen("/dev/random", "r")) == NULL) {
      printf("Error in opening random number device!\n");
      exit(EXIT_FAILURE);
    }
  }
  else if((ran_dev = fopen("/dev/urandom", "r")) == NULL) {
      printf("Error in opening random number device!\n");
      exit(EXIT_FAILURE);
  }
}

static int get_random(int sides) {
  unsigned int ret_value;

  if(ran_file || urandom) {
    if(!(fread(&ret_value, sizeof(unsigned int), 1, ran_dev) == 1)) {
      printf("Error in reading random device!\n");
      exit(EXIT_FAILURE);
    }
    return (int)(ret_value % sides);
  }
  if(use_rand)
    return (int)(( (float)rand() / (RAND_MAX + 1.0) ) * sides);
}

/* rolldie() - Rolls a single die
 *
 * Parameters: int *dice_nums - array of nums describing the different
 *                              aspects of the dice to be rolled
 * Returns: int - the result of the roll
 */
static int rolldie ( int *dice_nums ) {
  int index, result, total = 0;

  for( index = 0; index < dice_nums[NUM_DICE]; index++ ) {
    result = 1 + get_random(dice_nums[NUM_SIDES]);
    total += result;
  }

  total *= dice_nums[MULTIPLIER];
  total += dice_nums[MODIFIER];

  return total;
}

/* rolldice() - Rolls an amount of dice
 *
 * Parameters: int *dice_nums - array of nums describing the different
 *                              aspects of the dice to be rolled
 * Returns: int * - array of roll results
 */
/*@null@*/ int *rolldice (int *dice_nums) {
  int *totals, index;

  if((totals = malloc(dice_nums[NUM_ROLLS] * sizeof(int))) == NULL) 
    return NULL;

  for(index = 0; index < dice_nums[NUM_ROLLS]; index++)
  	totals[index] = 0;
    
  for(index = 0; index < dice_nums[NUM_ROLLS]; index++)
    if ((totals[index] = rolldie(dice_nums)) == 0) {
      free(totals);
      return NULL;
    }

  (void)sleep(SLEEP_TIME);

  return totals;
}

/* parse_string() - Parses a string for dice rolling attributes
 *
 * Parameters: char *dice_string - string to parse
 * Returns: int * - array of nums describing the different aspects of the 
 * dice to be rolled
 */
/*@null@*/ int *parse_string ( char *dice_string ) {
  int *dice_nums, index;
  char temp_char, *position;
  
  if((dice_nums = malloc ( DICE_ARRAY_SIZE * sizeof(int))) == NULL) return NULL;

  for( index = 0; index < DICE_ARRAY_SIZE; index++ ) 
    dice_nums[index] = 0;
 
  if( (position = strstr(dice_string, ROLL_IDENT)) != NULL ) {
    if(((sscanf(dice_string, "%d", &dice_nums[NUM_ROLLS])) < 1 ) 
       || (dice_nums[NUM_ROLLS] < 1)
       || (dice_nums[NUM_ROLLS] > MAXSHORT)) {
      free(dice_nums);
      return NULL;
    }
    dice_string = ++position;
  } else dice_nums[NUM_ROLLS] = 1;

  if( (position = strstr(dice_string, DICE_SIDES_IDENT)) != NULL ) {
    if(sscanf(dice_string, "%d", &dice_nums[NUM_DICE]) < 1 ) 
      dice_nums[NUM_DICE] = 1;
    else if ((dice_nums[NUM_DICE] < 0) || (dice_nums[NUM_DICE] > MAXSHORT)) {
      free(dice_nums);
      return NULL;
    }
    dice_string = ++position;
    if ((sscanf(dice_string, "%d", &dice_nums[NUM_SIDES]) < 1) ||
      ((dice_nums[NUM_SIDES] < 0) || (dice_nums[NUM_SIDES] > MAXSHORT))) {  
      if(*dice_string == '%')
        dice_nums[NUM_SIDES] = 100;
      else {
      free(dice_nums);
      return NULL;
      }
    }
  } else {
    free(dice_nums);
    return NULL;
  }
  
  if( (position = strstr(dice_string, MULTI_IDENT)) != NULL ) {
    dice_string = ++position;
    if(((sscanf(dice_string, "%d", &dice_nums[MULTIPLIER])) < 1) 
       || (dice_nums[MULTIPLIER] < 1)
       || (dice_nums[MULTIPLIER] > MAXSHORT)) {
      free(dice_nums);
      return NULL;
    }
  } else dice_nums[MULTIPLIER] = 1;

  if( ((position = strstr(dice_string, MODI_PLUS_IDENT)) != NULL) ||
      ((position = strstr(dice_string, MODI_MINUS_IDENT))!= NULL)) { 
    dice_string = position;
    temp_char = *dice_string;
    if(((sscanf(++dice_string, "%d", &dice_nums[MODIFIER])) < 1)
       || (dice_nums[MODIFIER] < 0)
       || (dice_nums[MODIFIER] > MAXSHORT)) {
      free(dice_nums);
      return NULL;
    }
    if( temp_char == '-' )
      dice_nums[MODIFIER] = 0 - dice_nums[MODIFIER];
  } else dice_nums[MODIFIER] = 0;

  return dice_nums;
}

